<?php

namespace CADSUP\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\Framework\Report\ReportService;

class Sup02400CalendarioDeDiasNaoUteisController extends \Zeedhi\Framework\Controller\Report 
{

	protected $reportService;
	protected $dataSourceName;
    protected $dataSourceManager;
    protected $crudService;
    protected $birtStrategy;
    protected $environment;
    protected $newCode;
    protected $cadSupUtil;
    
    public function __construct(ReportService $reportService, $crudService, $dataSourceManager, $connection, $service, $birtStrategy, $environment, $newCode, $cadSupUtil)
    {
        parent::__construct($reportService);
    	$this->crudService       = $crudService;
    	$this->dataSourceManager = $dataSourceManager;
    	$this->con               = $connection;
    	$this->service = $service;
    	$this->birtStrategy = $birtStrategy;
    	$this->environment = $environment;
    	$this->newCode = $newCode;
    	$this->cadSupUtil = $cadSupUtil;
    }
    
    protected $reportMapping = array ();
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $row['NRORG'] = $this->environment->getNrOrg();
        
        if($row['CDFILIAL'] == ''){
            $row['CDFILIAL'] = 'T';
        }
        
        if($row['CDFILIAL'] != 'T'){
            $nrlistaForn = $this->cadSupUtil->insertLista($row['CDFILIAL']);
            $row['CDFILIAL'] = "_NRLISTA=".$nrlistaForn;
        }
        $this->birtStrategy->setReportFormat($row['QR2TYPE']);
        $this->stNomeRel = $this->getReportName($row);
        $this->reportMapping[$this->stNomeRel] =
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'CDFILIAL'      => 'L_CDFILIAL',
                'DTINICIAL'     => 'P_DTINICIAL',
                'DTFINAL'       => 'P_DTFINAL',
                'CDGRUPFILI'    => 'P_CDGRUPFILI',
                'NRORG'         => 'P_NRORG'
            ),
        );
    }
    
    public function save($request, $response)
    {
        try{
            $rows = $request->getDataSet()->getRows();
            foreach($rows as $key => $row){
                $this->con->beginTransaction(); 
                $rowsToSave = array();
                $calendar = $row['CALENDAR'];
                
                $condition = array(
                    array(
                        "columnName"=>'CDFILIAL',
                        "operator"  => "=",
                        "value"     => $row['CDFILIAL']
                    ),
                    array(
                        "columnName"=>$row['TABS'][$calendar]['DTFERI'],
                        "operator"  => "=",
                        "value"     => $row[$row['TABS'][$calendar]['DTFERI']]
                    )
                );
                $filterCriteria = new FilterCriteria($row['TABS'][$calendar]['TABFERI'], $condition);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                if(empty($dataSet->getRows()) && $row['__is_new'] || !empty($dataSet->getRows()) && !$row['__is_new']){
                    array_push($rowsToSave, $row);
                    $newDataSet = new DataSet(null , $rowsToSave);
                    $newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
                    $toSave = $this->crudService->getRowsToSave($newRequest, $row['TABS'][$row['CALENDAR']]['TABFERI']);
                    $this->crudService->save($row['TABS'][$calendar]['TABFERI'], $toSave);
                }
                $this->con->commit();
            }
            $response->addMessage(new Message("successOperation"));
		} catch (\Exception $e) {
		    $this->con->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
    }
    
    public function verificaDia($request, $response) 
    {
    	$currentRows = $request->getDataset()->getRows();
        $newDataSet = array();
    	$resp = array();
        $row = array();
        $dtSetArray = array();
        foreach($currentRows as $currentRow) {
            $condition = array(
                array(
                    "columnName"=>'CDFILIRECE',
                    "operator"  => "=",
                    "value"     => $currentRow['CDFILIAL']
                ),
                array(
                    "columnName"=>'DTENTRSAID',
                    "operator"  => "=",
                    "value"     => $currentRow['DTDIA']
                ),
                array(
                    "columnName"=>'NRORG',
                    "operator"  => "=",
                    "value"     => $currentRow['NRORG']
                )
            );

            $filterCriteria = new FilterCriteria('notafiscal', $condition);
            $dataSet = $this->dataSourceManager->findBy($filterCriteria);

            if (!empty($dataSet->getRows())){
                $newDataSet['ERRORMSG'] = 'EXISTE_NFE';
            } else {
                $condition[0]['columnName']='CDFILIEMIT';
                $filterCriteria = new FilterCriteria('notafiscal', $condition);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                if (!empty($dataSet->getRows())){
                    $newDataSet['ERRORMSG'] = 'EXISTE_NFS';
                } else{
                    $condition = array(
                        array(
                            "columnName"=>'CDFILIAL',
                            "operator"  => "=",
                            "value"     => $currentRow['CDFILIAL']
                        ),
                        array(
                            "columnName"=>'NRORG',
                            "operator"  => "=",
                            "value"     => $currentRow['NRORG']
                        )
    
                    );
    
                    $filterCriteria = new FilterCriteria('paramplaneja', $condition);
                    $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                    if (!empty($dataSet->getRows()) && $dataSet->getRows()[0]['IDCARDIAUTIL']=='S') {
                        $condition = array(
                            array(
                                "columnName"=>'CDFILIAL',
                                "operator"  => "=",
                                "value"     => $currentRow['CDFILIAL']
                            ),
                            array(
                                "columnName"=>'DTCARDAPIO',
                                "operator"  => "=",
                                "value"     => $currentRow['DTDIA']
                            ),
                            array(
                                "columnName"=>'NRORG',
                                "operator"  => "=",
                                "value"     => $currentRow['NRORG']
                            )
                        );
                        $filterCriteria = new FilterCriteria('cardadia_calendario', $condition);
                        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                        $rows = $dataSet->getRows();
                       
                        foreach ($rows as $row) {
                            if (array_key_exists('NRRETIRADA',$row)){
                                $newDataSet['ERRORMSG'] = 'EXISTE_RETIRADA';
                            }
                        }
                        foreach ($rows as $row) {
                            if ($row['NRCARDAPIO']){
                                $DATADIA = DateTime::createFromFormat('d/m/Y H:i:s',$row['DTCARDAPIO']);
                                $DATAPROC = DateTime::createFromFormat('d/m/Y H:i:s',$currentRow['DTPROCE']);
                                if($DATADIA<$DATAPROC){
                                    $newDataSet['ERRORMSG'] = 'EXISTE_CARDAPIO';
                                } else {
                                    $newDataSet['MSG'] = 'CONFIRM_EXCL_CARD';
                                    $newDataSet['TYPE']         = "industrial";
                                    $newDataSet['CDFILIAL']     = $currentRow['CDFILIAL'];
                                    $newDataSet['DTCARDAPIO']   = $currentRow['DTDIA'];
                                    $newDataSet['DTCARDIAPME']  = $currentRow['DTDIA'];
                                    $newDataSet['DTPROCE']      = $currentRow['DTPROCE'];
                                    $newDataSet['CDFILCENTPME'] = null;
                                    $newDataSet['__is_new']     = false;
                                }
                            }
                        }
                        $condition = array(
                            array(
                                "columnName"=>'CDFILIAL',
                                "operator"  => "=",
                                "value"     => $currentRow['CDFILIAL']
                            ),
                            array(
                                "columnName"=>'DTCARDIAPME',
                                "operator"  => "=",
                                "value"     => $currentRow['DTDIA']
                            ),
                            array(
                                "columnName"=>'NRORG',
                                "operator"  => "=",
                                "value"     => $currentRow['NRORG']
                            )
                        );
                        $filterCriteria = new FilterCriteria('cardiapme_calendario', $condition);
                        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                        $rows = $dataSet->getRows();
                        foreach ($rows as $row) {
                            if ($row['DTCARDIAPME']){
                                $DATADIA = DateTime::createFromFormat('d/m/Y H:i:s',$row['DTCARDIAPME']);
                                $DATAPROC = DateTime::createFromFormat('d/m/Y H:i:s',$currentRow['DTPROCE']);
                                if($DATADIA<$DATAPROC){ 
                                    $newDataSet['ERRORMSG'] = 'EXISTE_MERENDA';
                                    
                                } else {
                                    $newDataSet['MSG'] = 'CONFIRM_EXCL_MERENDA';
                                    $newDataSet['TYPE']         = "merenda";
                                    $newDataSet['CDFILCENTPME'] = $row['CDFILCENTPME'];
                                    $newDataSet['CDFILIAL']     = $currentRow['CDFILIAL'];
                                    $newDataSet['DTCARDAPIO']   = $currentRow['DTDIA'];
                                    $newDataSet['DTCARDIAPME']  = $currentRow['DTDIA'];
                                    $newDataSet['DTPROCE']      = $currentRow['DTPROCE'];
                                    $newDataSet['NRCARDIAPME']  = $row['NRCARDIAPME'];
                                    $newDataSet['NRCARDPME']    = $row['NRCARDPME'];
                                    $newDataSet['__is_new']     = false;
                                }
                            }
                        }
                        
                    }
                }
            }
            
            array_push($resp, [$currentRow['TABCAL'] => $newDataSet]);
        }

        $response->addDataSet(new DataSet("resp", $resp));
    }

    public function delete($request, $response)
    {
        $rows = $request->getDataset()->getRows();
        try{
            foreach ($rows as $row) {
                $dataSet = new DataSet($row['TABS'][$row['TABCAL']]['TABFERI'],array($row));
                $this->crudService->delete($response,$dataSet);
            }
            $response->addMessage(new Message('successOperation'));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
    }
    
    public function deleteCardapio($request, $response)
    {
        $row = $request->getRow();

        try{
            //Se merenda
            if($row['TYPE'] == 'merenda'){
                //Se for central
                if($row['CDFILIAL'] == $row['CDFILCENTPME']){
                    $selectRetornos = $this->service->selectCardUnidadeCentralMerenda($row);
                    foreach($selectRetornos as $selectRetorno){
                        $this->service->deleteCardUnidadeCentralMerenda($selectRetorno);
                    }
                    $this->service->deleteCardUnidadeCentralAposLacoMerenda($row);
                }
                else {
                    $selectRetornos = $this->service->selectCardUnidadeCentralMerenda($row);
                    foreach($selectRetornos as $selectRetorno){
                        $this->service->deleteCardUnidadeMerenda($selectRetorno);
                    }
                    $this->service->deleteCardUnidadeAposLacoMerenda($row);
                }
            } else if($row['TYPE'] == 'industrial'){
                $this->service->deleteCardIndustrial($row);
            }
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}

    }
    
    public function calculoCompras($request, $response)
    {
        $row = $request->getDataSet()->getRows()[0];
        $perientr = $row['ALLPERIENTR'] == 'Y' ? 'T' : $row['PERIODOS'];
        
        $query = "SELECT 
                    DISTINCT PERFILPROD.CDPERIENTR, 
                    PERIENTR.IDTIPODIA 
                FROM 
                    PERFILPROD, 
                    PERIENTR 
                WHERE 
                    (PERFILPROD.CDFILIAL = :P_CDFILIAL) 
                    AND 
                        (
                            (INSTR(:P_CDPERIENTR, PERFILPROD.CDPERIENTR) <> 0) 
                        OR 
                            :P_CDPERIENTR = 'T') 
                    AND 
                        (PERFILPROD.CDPERIENTR = PERIENTR.CDPERIENTR)";
        $stmt = $this->con->prepare($query);
        $stmt->execute(array(
		 	"P_CDPERIENTR"  => $perientr,
		 	"P_CDFILIAL"    => $row['CDFILIAL']
		));
		$resultados = $stmt->fetchAll(\PDO::FETCH_ASSOC);
		
		if($resultados){
    		try{
        		foreach($resultados as $resultado){
        			
        			$params['P_CDFILIAL'] = $row['CDFILIAL'];
        			$params['P_CDPERIENTR'] = $resultado['CDPERIENTR'];
        			$params['P_DTPROCESSA'] = $row['DTPROCESSA'];
        			$nome = $resultado['IDTIPODIA'] == 'R' ? 'SEMANAL_RECORRENTE_DTENTR' : 'CALC_DATA_ENTR_MES';
        			if($row['ALLMONTHSARESELECTED']){
        				$params['P_DTINICIAL'] = $row['DATAS'][0];
        				$params['P_NRMESES'] = "12";
        				$this->executeCalculoProcedure($params, $nome);
        			}else{
        				foreach($row['DATAS'] as $mes){
        					$params['P_DTINICIAL'] = $mes;
        					$params['P_NRMESES'] = "1";
        					$this->executeCalculoProcedure($params, $nome);
        				}
        			}
        		}
    		} catch (\Exception $e) {
    			$response->setError(new Error($e->getMessage(), 500));
    		}
    	}
    }
    
    public function exportarCalendario($request, $response)
    {
        
        $row            = $request->getDataSet()->getRows()[0];
        $ano            = $row['ANO'];
        $acao           = $row['ACAO'];
        $meses          = $row['MESES'];
        $allfilial      = $row['ALLFILIAL'];
        $tipoCalendario = $row['CALENDARIO'];
        $cdoperador     = $row['CDOPERADOR'];
        $cdfilial       = $row['CDFILIORIGEM'];
        $filiaisDestino = $row['CDFILIDESTINO'];
        $grupfili       = $row['GRUPFILI'] ? $row['GRUPFILI'] : 'T';

        if($allfilial){
            $query = "SELECT DISTINCT FILIAL.CDFILIAL FROM FILIAL, FILIOPER 
                      WHERE FILIAL.CDFILIAL <> :P_CDFILIAL 
                        AND FILIOPER.CDOPERADOR = :P_CDOPERADOR 
                        AND FILIAL.CDFILIAL = FILIOPER.CDFILIAL 
                        AND (
                            (:P_CDGRUPFILI = 'T') 
                            OR (
                                EXISTS 
                                (
                                    SELECT 1 
                                    FROM FILGRUPFILI 
                                    WHERE FILGRUPFILI.CDFILIAL = FILIAL.CDFILIAL 
                                        AND FILGRUPFILI.CDGRUPFILI = :P_CDGRUPFILI
                                )
                            )
                        ) 
                       ORDER BY FILIAL.CDFILIAL";
            $stmt = $this->con->prepare($query);
            $stmt->execute(array(
    		 	'P_CDFILIAL'   => $cdfilial,
    		 	'P_CDOPERADOR' => $cdoperador,
    		 	'P_CDGRUPFILI' => $grupfili
    		));
		    $unidades = $stmt->fetchAll(\PDO::FETCH_ASSOC);
		    
        }else{
            $unidades = $filiaisDestino;
        }
        try{
            foreach ($unidades as $unidade) {
                $unidade = is_array($unidade) ? $unidade['CDFILIAL'] : $unidade;
                
                $DTPROCESSA = new DateTime($this->findDTPROCESSA($unidade, $cdoperador, $grupfili));
                $DTPROCESSA = $DTPROCESSA->format('d/m/Y');
                if($DTPROCESSA){
                    foreach ($tipoCalendario as $calendario) {
                            $this->con->beginTransaction();
                            
                            $datas = $this->CRUDCalendario('SELECT_DATA', $DTPROCESSA, '', $cdfilial, $calendario, $meses, $ano);
                            
                            if($acao === 'SOB'){
                                $this->CRUDCalendario('DELETE', '', '', $unidade, $calendario, $meses, $ano);
                            }
                            foreach ($datas as $feriado) {
                                if($acao === 'ACR'){
                                    $existe = $this->CRUDCalendario('SELECT_EXISTE', $feriado['DATA'], '', $unidade, $calendario, $meses, $ano);
                                    
                                    if(!$existe){
                                        $this->CRUDCalendario('INSERT', $feriado['DATA'], $feriado['DESCRICAO'], $unidade, $calendario, $meses, $ano);
                                    }
                                }else{
                                    $this->CRUDCalendario('INSERT', $feriado['DATA'], $feriado['DESCRICAO'], $unidade, $calendario, $meses, $ano);
                                }
                            }
                            $this->con->commit();
                        }
                    }
                }
            $response->addMessage(new Message("successOperation"));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
			$this->con->rollback();
		}
    }

    
    public function findDTPROCESSA($cdfilial, $cdoperador, $grupfili)
    {
        $query = "SELECT DISTINCT PF.CDFILIAL, TO_CHAR(DECODE(PG.IDUTILDTPROC,'S', PF.DTPROCESSA, SYSDATE), 'YYYY-MM-DD') AS DTPROCESSA
                  FROM PARAMFILIAL PF, FILIOPER FO, PARAMGERAL PG
                 WHERE ((INSTR(:P_CDFILIAL, PF.CDFILIAL) <> 0) OR (:P_CDFILIAL = 'T'))
                   AND FO.CDOPERADOR = :P_CDOPERADOR
                   AND PF.CDFILIAL = FO.CDFILIAL
                   AND ((:P_CDGRUPFILI = 'T') OR EXISTS (SELECT 1
                                                           FROM FILGRUPFILI FG
                                                          WHERE FG.CDGRUPFILI = :P_CDGRUPFILI
                                                            AND PF.CDFILIAL = FG.CDFILIAL))";
        
        try{
            $stmt = $this->con->prepare($query);
            $stmt->execute(array(
                'P_CDFILIAL'   => $cdfilial,
                'P_CDOPERADOR' => $cdoperador,
                'P_CDGRUPFILI' => $grupfili
            ));

    		$data = $stmt->fetch(\PDO::FETCH_ASSOC);
    		if($data){
		        return $data['DTPROCESSA'];
    		}else{
    		    return false;
    		}
        }catch(\Exception $e) {
            
			throw new \Exception();
        }
        
    }
    
    public function CRUDCalendario($crud, $data, $description, $cdfilial, $tipoCalendario, $meses, $ano)
    {
        switch ($tipoCalendario) {
            case 'filial':
                $tableName = 'FERIFILI';
                $dateFieldName = 'DTFERIFILI';
                $descriptionFieldName = 'DSFERIFILI';
            break;
            case 'compras':
                $tableName = 'FERICOMP';
                $dateFieldName = 'DTFERICOMP';
                $descriptionFieldName = 'DSFERICOMP';
            break;
            case 'cardapio':
                $tableName = 'FERICARD';
                $dateFieldName = 'DTFERICARD';
                $descriptionFieldName = 'DSFERICARD';
            break;
            case 'bancario':
                $tableName = 'FERIBANC';
                $dateFieldName = 'DTFERIBANC';
                $descriptionFieldName = 'DSFERIBANC';
            break;
        }
    
        switch ($crud) {
            case 'SELECT_EXISTE':
                
                $query = "SELECT " . $descriptionFieldName ." 
                      FROM " . $tableName . " 
                      WHERE CDFILIAL = '". $cdfilial ."' 
                        AND " . $dateFieldName . " = '" . $data . "'";
            break;
            case 'SELECT_DATA':
                
                $query = "SELECT " . $dateFieldName ." AS data, ". $descriptionFieldName." AS descricao
                          FROM " . $tableName . " 
                          WHERE CDFILIAL = '". $cdfilial ."' 
                            AND " . $dateFieldName . " > '" . $data . "'
                            AND INSTR(:meses, TO_CHAR(".$dateFieldName.", 'MM')) <> 0
                            AND " . $dateFieldName . " BETWEEN '01/01/" . $ano . "' AND '31/12/" . $ano ."'";
            break;
            case 'INSERT':
                
                $query = "INSERT INTO " . $tableName . " 
                            (CDFILIAL, ". $dateFieldName .", " . $descriptionFieldName . "  ) 
                        VALUES 
                            ('".$cdfilial."', '" . $data . "', '".$description."')";
            break;
            case 'DELETE':
                
                $query = "DELETE FROM " . $tableName . " 
                          WHERE CDFILIAL = '" . $cdfilial . "'  
                            AND ". $dateFieldName ." BETWEEN '01/01/" . $ano . "' AND '31/12/" . $ano . "'";
            break;
        }
        try{
            $stmt = $this->con->prepare($query);
            if($crud == 'SELECT_DATA'){
                $stmt->execute(array(
                    "meses" => $meses
                ));
            }else{
                $stmt->execute();
            }
            if($crud != 'INSERT' && $crud != 'UPDATE' && $crud != 'DELETE'){
    		    return $stmt->fetchAll(\PDO::FETCH_ASSOC);
            }
        } catch (\Exception $e) {
			throw new \Exception();
		}
    }
    
    public function executeCalculoProcedure($params, $name)
    {
		$procedure = new StoredProcedure($this->con, $name);
        $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT,  $params['P_CDFILIAL'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPERIENTR',  Param::PARAM_INPUT,  $params['P_CDPERIENTR'],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL',   Param::PARAM_INPUT,  $params['P_DTINICIAL'],    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRMESES',     Param::PARAM_INPUT,  $params['P_NRMESES'],      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTPROCESSA',  Param::PARAM_INPUT,  $params['P_DTPROCESSA'],   Param::PARAM_TYPE_STR));
        $response = $procedure->execute();
        return $response;
	}
	
	public function verificaErroDia(DTO\Request $request, DTO\Response $response) {
		try {
            $row = $request->getRow();
            
            $erroRequisicao = $this->service->getErrosRequisicao($row);
            if ($erroRequisicao){
                throw new \Exception("EXISTE_REQUISICAO");
            } 
            
            $erroEfetivos = $this->service->getErrosEfetivos($row);
            if ($erroEfetivos){
                throw new \Exception("EXISTE_EFETIVO");
            }

		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
}
